precision highp float;
varying highp vec2 textureCoordinate;

uniform sampler2D inputImageTexture;
uniform sampler2D inputImageTexture2;
uniform sampler2D inputImageTexture3;
uniform sampler2D filterTexture1;
uniform sampler2D filterTexture2;
uniform sampler2D blurTexture;
uniform highp vec2 inputSize;
uniform float mRepeatPercent[8];
uniform float mBlendMatrixArray[160];
uniform float mFrameRatio;
uniform int mRepeatMode;
uniform int mFrameShowCount;
uniform int mMaskingType;
uniform float mFilterIndexArray[10];
uniform float mMaskIndexArray[10];
uniform float mBlurIndexArray[10];
uniform float mLimitPosition[40];
const float threshold = 0.001;


vec4 lookupBlend(vec4 textureColor, float alpha, sampler2D filterTexture) {
    highp float blueColor = textureColor.b * 63.0;
    highp vec2 quad1;
    quad1.y = floor(floor(blueColor) / 8.0);
    quad1.x = floor(blueColor) - (quad1.y * 8.0);
    highp vec2 quad2;
    quad2.y = floor(ceil(blueColor) / 8.0);
    quad2.x = ceil(blueColor) - (quad2.y * 8.0);
    highp vec2 texPos1;
    texPos1.x = (quad1.x * 0.125) + 0.5/512.0 + ((0.125 - 1.0/512.0) * textureColor.r);
    texPos1.y = (quad1.y * 0.125) + 0.5/512.0 + ((0.125 - 1.0/512.0) * textureColor.g);
    highp vec2 texPos2;
    texPos2.x = (quad2.x * 0.125) + 0.5/512.0 + ((0.125 - 1.0/512.0) * textureColor.r);
    texPos2.y = (quad2.y * 0.125) + 0.5/512.0 + ((0.125 - 1.0/512.0) * textureColor.g);
    lowp vec4 newColor1 = texture2D(filterTexture, texPos1);
    lowp vec4 newColor2 = texture2D(filterTexture, texPos2);
    lowp vec4 newColor = mix(newColor1, newColor2, fract(blueColor));
    return mix(textureColor, vec4(newColor.rgb, textureColor.w), alpha);
}

bool isOutside(vec2 uv){
    return uv.x <0.0 || uv.x >1.0 ||uv.y <0.0 || uv.y >1.0;
}

bool isOutsidePosition(vec2 uv, vec4 limitLocation){
    return uv.x < limitLocation.x || uv.x > limitLocation.y ||uv.y <limitLocation.z || uv.y >limitLocation.w;
}

vec4 getRepeatXModeColor(float imageRatio) {
    vec4 frameColor;

    if (textureCoordinate.y < 0.5){
        float repeatPercent = mRepeatPercent[5] - mRepeatPercent[4];
        if (textureCoordinate.x < mRepeatPercent[4] / imageRatio){
            frameColor = texture2D(inputImageTexture2, vec2(textureCoordinate.x*imageRatio, textureCoordinate.y));
        } else if (textureCoordinate.x < 1.0 - (1.0 - mRepeatPercent[5]) / imageRatio){
            float count = floor((textureCoordinate.x - mRepeatPercent[4] / imageRatio)/(repeatPercent / imageRatio));
            frameColor = texture2D(inputImageTexture2, vec2((textureCoordinate.x - repeatPercent *count /imageRatio) * imageRatio, textureCoordinate.y));
        } else {
            frameColor = texture2D(inputImageTexture2, vec2(textureCoordinate.x * imageRatio - (imageRatio - 1.0), textureCoordinate.y));
        }
    } else {
        float repeatPercent = mRepeatPercent[7] - mRepeatPercent[6];
        if (textureCoordinate.x < mRepeatPercent[6] / imageRatio){
            frameColor = texture2D(inputImageTexture2, vec2(textureCoordinate.x*imageRatio, textureCoordinate.y));
        } else if (textureCoordinate.x < 1.0 - (1.0 - mRepeatPercent[7]) / imageRatio){
            float count = floor((textureCoordinate.x - mRepeatPercent[6] / imageRatio)/(repeatPercent / imageRatio));
            frameColor = texture2D(inputImageTexture2, vec2((textureCoordinate.x - repeatPercent *count /imageRatio) * imageRatio, textureCoordinate.y));
        } else {
            frameColor = texture2D(inputImageTexture2, vec2(textureCoordinate.x * imageRatio - (imageRatio - 1.0), textureCoordinate.y));
        }
    }

    return frameColor;
}


vec4 getRepeatYModeColor(float imageRatio) {
    vec4 frameColor;

    if (textureCoordinate.x < 0.5){
        float repeatPercent = mRepeatPercent[1] - mRepeatPercent[0];
        if (textureCoordinate.y < mRepeatPercent[0] * imageRatio){
            frameColor = texture2D(inputImageTexture2, vec2(textureCoordinate.x, textureCoordinate.y/imageRatio));
        } else if (textureCoordinate.y < 1.0 -  (1.0 - mRepeatPercent[1])  * imageRatio){
            float count = floor((textureCoordinate.y - mRepeatPercent[0] * imageRatio)/(repeatPercent * imageRatio));
            frameColor = texture2D(inputImageTexture2, vec2(textureCoordinate.x, (textureCoordinate.y - repeatPercent *count * imageRatio)/imageRatio));
        } else {
            frameColor = texture2D(inputImageTexture2, vec2(textureCoordinate.x, textureCoordinate.y/imageRatio - (1.0/imageRatio -1.0)));
        }
    } else {
        float repeatPercent = mRepeatPercent[3] - mRepeatPercent[2];
        if (textureCoordinate.y < mRepeatPercent[2] * imageRatio){
            frameColor = texture2D(inputImageTexture2, vec2(textureCoordinate.x, textureCoordinate.y/imageRatio));
        } else if (textureCoordinate.y < 1.0 -  (1.0 - mRepeatPercent[3])  * imageRatio){
            float count = floor((textureCoordinate.y - mRepeatPercent[2] * imageRatio)/(repeatPercent * imageRatio));
            frameColor = texture2D(inputImageTexture2, vec2(textureCoordinate.x, (textureCoordinate.y - repeatPercent *count * imageRatio)/imageRatio));
        } else {
            frameColor = texture2D(inputImageTexture2, vec2(textureCoordinate.x, textureCoordinate.y/imageRatio - (1.0/imageRatio -1.0)));
        }

    }

    return frameColor;
}

vec4 getRepeatXYModeColor(float imageRatio) {
    if (imageRatio >1.0){
        return getRepeatXModeColor(imageRatio);
    } else {
        return getRepeatYModeColor(imageRatio);
    }
}

mat4 getCurrentBlendMatrix(int index){
    int currentMatrixIndex = index * 16;
    mat4 blendMatrix = mat4(
    mBlendMatrixArray[0+ currentMatrixIndex], mBlendMatrixArray[1+ currentMatrixIndex], mBlendMatrixArray[2+ currentMatrixIndex], mBlendMatrixArray[3+ currentMatrixIndex],
    mBlendMatrixArray[4+ currentMatrixIndex], mBlendMatrixArray[5+ currentMatrixIndex], mBlendMatrixArray[6+ currentMatrixIndex], mBlendMatrixArray[7+ currentMatrixIndex],
    mBlendMatrixArray[8+ currentMatrixIndex], mBlendMatrixArray[9+ currentMatrixIndex], mBlendMatrixArray[10+ currentMatrixIndex], mBlendMatrixArray[11+ currentMatrixIndex],
    mBlendMatrixArray[12+ currentMatrixIndex], mBlendMatrixArray[13+ currentMatrixIndex], mBlendMatrixArray[14+ currentMatrixIndex], mBlendMatrixArray[15+ currentMatrixIndex]
    );
    return blendMatrix;
}

vec4 getCurrentLimitPosition(int index){
    int currentPositionIndex = index * 4;
    vec4 currentLimitPosition = vec4(mLimitPosition[0+currentPositionIndex], mLimitPosition[1+currentPositionIndex],
    mLimitPosition[2+currentPositionIndex], mLimitPosition[3+currentPositionIndex]);
    return currentLimitPosition;
}

vec4 getSrcColor(int index) {
    vec4 coordinate = vec4(textureCoordinate, 0., 1.);
    mat4 mBlendMatrix = getCurrentBlendMatrix(index);
    coordinate= mBlendMatrix * coordinate;
    vec4 currentLimitPosition = getCurrentLimitPosition(index);
    vec4 srcColor;
    if (isOutsidePosition(textureCoordinate, currentLimitPosition) || isOutside(coordinate.xy)){
        return vec4(0.0);
    } else {
        srcColor = mBlurIndexArray[index] > threshold ? texture2D(blurTexture, coordinate.xy) : texture2D(inputImageTexture, coordinate.xy);
    }
    if (mMaskingType != 0){
        vec4 maskingColor = texture2D(inputImageTexture3, textureCoordinate);
        if(abs(mMaskIndexArray[index])< threshold){
            float r = (255.0 - 51.0 * float(index))/255.0;
            if (maskingColor.a < threshold || abs(maskingColor.r - r) > threshold){
                return vec4(0.0);
            }

        } else if (abs(mMaskIndexArray[index] - 2.0)< threshold){
            srcColor = srcColor * maskingColor.a ;
        }
    }

    if (abs(mFilterIndexArray[index] - 1.0)< threshold){
        srcColor=  lookupBlend(srcColor, 1.0, filterTexture1);
    } else if (abs(mFilterIndexArray[index] - 2.0)< threshold){
        srcColor = lookupBlend(srcColor, 1.0, filterTexture2);
    }
    return srcColor;
}

void drawResultColorNoRepeat() {
    vec4 frameColor = texture2D(inputImageTexture2, textureCoordinate);
    vec4 resultColor =vec4(0);
    for (int i =0;i<mFrameShowCount;i++){
        vec4 srcColor = getSrcColor(i);
        resultColor = srcColor  + resultColor * (1.0 - srcColor.a);
        //        resultColor = srcColor * (1.0-resultColor.a)  + resultColor ;
    }
    gl_FragColor = resultColor * (1.0 - frameColor.a)  + frameColor;
    //    gl_FragColor = resultColor * 0.8  + frameColor*0.5 ;
}

void drawResultColorRepeat() {
    vec4 srcColor = getSrcColor(0);
    float imageRatio = inputSize.x/inputSize.y;
    vec4 frameColor;
    if (mRepeatMode  == 1){
        frameColor = getRepeatXYModeColor(imageRatio);
    } else if (mRepeatMode  == 2){
        frameColor = getRepeatXModeColor(imageRatio);
    } else {
        frameColor = getRepeatYModeColor(imageRatio);
    }
    gl_FragColor = srcColor * (1.0 - frameColor.a)  + frameColor;

}

void main(){
    if (mRepeatMode  == 0){
        drawResultColorNoRepeat();
    } else {
        drawResultColorRepeat();
    }
}
