////// Fragment Shader
precision highp float;
varying vec2 textureCoordinate;
uniform sampler2D inputImageTexture;
uniform sampler2D inputImageTexture2;
const mediump vec3 satLuminanceWeighting = vec3(0.2126, 0.7152, 0.0722);
const mediump vec3 luminanceWeighting = vec3(0.3, 0.3, 0.3);

uniform int frontBackType;
uniform float shadows;
uniform float highlights;
uniform float contrast;
uniform float fadeAmount;
uniform float saturation;
uniform float warmth;
uniform float green;
uniform float vibrance;
uniform float ambiance;
uniform float vignette;
uniform float convex;
uniform float brightness;

vec4 aberrate(sampler2D source, vec2 uv, float amount) {
    return texture2D(source, 0.5 + uv / sqrt(1.0 + amount * dot(uv, uv)));
}

vec3 rgb2hsv(vec3 c) {
    vec4 K = vec4(0.0, -1.0 / 3.0, 2.0 / 3.0, -1.0);
    vec4 p = c.g < c.b ? vec4(c.bg, K.wz) : vec4(c.gb, K.xy);
    vec4 q = c.r < p.x ? vec4(p.xyw, c.r) : vec4(c.r, p.yzx);
    float d = q.x - min(q.w, q.y);
    float e = 1.0e-10;
    return vec3(abs(q.z + (q.w - q.y) / (6.0 * d + e)), d / (q.x + e), q.x);
}

vec3 hsv2rgb(vec3 c) {
    vec4 K = vec4(1.0, 2.0 / 3.0, 1.0 / 3.0, 3.0);
    vec3 p = abs(fract(c.xxx + K.xyz) * 6.0 - K.www);
    return c.z * mix(K.xxx, clamp(p - K.xxx, 0.0, 1.0), c.y);
}

vec3 highShade(vec4 result, float highlights, float dHighlights, float shadows, float dShadows, float srcBrightness) {
    float dh = (highlights + dHighlights) / 3.0;
    float ds = (shadows + dShadows) / 1.5;
    float d = -dh * pow(srcBrightness, 3.0);
    float mag = d * 3.045;
    float exppower = 1.0 + abs(mag);
    if (mag < 0.0) {
        exppower = 1.0 / exppower;
    }
    result.r = 1.0 - pow((1.0 - result.r), exppower);
    result.g = 1.0 - pow((1.0 - result.g), exppower);
    result.b = 1.0 - pow((1.0 - result.b), exppower);
    if (ds > 0.) {
        float tempDs = 1.0 - srcBrightness;
        vec4 whiteColor = vec4(1.0);
        vec4 tempResult = whiteColor - ((whiteColor - result) * (whiteColor - result * ds * 2.0 * tempDs));
        return tempResult.rgb;
    } else {
        d = ds - ds * srcBrightness * srcBrightness;
        mag = d * 3.045;
        exppower = 1.0 + abs(mag);
        if (mag < 0.0) {
            exppower = 1.0 / exppower;
        }
        result.r = 1.0 - pow((1.0 - result.r), exppower);
        result.g = 1.0 - pow((1.0 - result.g), exppower);
        result.b = 1.0 - pow((1.0 - result.b), exppower);
    }
    return result.rgb;
}

float easeInOutSigmoid(float value, float strength) {
    if (value > 0.5) {
        return 1.0 - pow(2.0 - 2.0 * value, 1.0 / (1.0 - strength)) * 0.5;
    } else {
        return pow(2.0 * value, 1.0 / (1.0 - strength)) * 0.5;
    }
}

vec3 fadeAdjust(vec3 color, float fadeVal) {
    return (color * (1.0 - fadeVal)) + ((color + (vec3(-0.9772) * pow(vec3(color), vec3(3.0)) + vec3(1.708) * pow(vec3(color), vec3(2.0)) + vec3(-0.1603) * vec3(color) + vec3(0.2878) - color * vec3(0.9))) * fadeVal);
}

void main() {
    vec4 source = texture2D(inputImageTexture, textureCoordinate);
    if(source.a ==0.0){
        gl_FragColor =  vec4(0);
        return;
    }
    vec4 maskColor = texture2D(inputImageTexture2, textureCoordinate);
    bool isFront = frontBackType == 1;
    float alpha = isFront ? maskColor.a : 1.0 - maskColor.a;

    float shadowsResult =  shadows * alpha ;
    float highlightsResult = highlights * alpha;
    float contrastResult = contrast * alpha;
    float fadeAmountResult = fadeAmount * alpha;
    float saturationResult =  saturation * alpha;
    float warmthResult = warmth * alpha;
    float greenResult = green * alpha;
    float vibranceResult = vibrance * alpha;
    float ambianceResult = ambiance * alpha;
    float brightnessResult = brightness * alpha;

    vec4 result = vec4(0.0);
    const float toolEpsilon = 0.005;

    result = source;
    float dHighlights = 0.;
    float dShadows = 0.;
    float dSaturation = 0.;

    //ambiance
    if (ambianceResult > 0.05) {
        dHighlights = ambianceResult / 3.0;
        dShadows = ambianceResult / 3.0;
        dSaturation = ambianceResult / 12.0;
    } else if (ambianceResult < -0.05) {
        dHighlights = ambianceResult / 6.0;
        dShadows = ambianceResult / 3.0;
        dSaturation = ambianceResult / 15.0;
    }


    // contrast
    if (abs(contrastResult) > toolEpsilon) {
        float dc = contrastResult * 2.;
        dHighlights -= dc;
        dShadows -= dc;
    }

    // highlights and shadows
    if ((abs(highlightsResult + dHighlights) > toolEpsilon || abs(shadowsResult + dShadows) > toolEpsilon)) {
        float srcBrightness = dot(result.rgb, luminanceWeighting);
        result.rgb = highShade(result, highlightsResult, dHighlights, shadowsResult, dShadows, srcBrightness);
    }

    // brightness
    if (brightnessResult > toolEpsilon) {
        float mag = brightnessResult * 1.045;
        float exppower = 1.0 + abs(mag);
        result.r = 1.0 - pow((1.0 - result.r), exppower);
        result.g = 1.0 - pow((1.0 - result.g), exppower);
        result.b = 1.0 - pow((1.0 - result.b), exppower);
    } else if (brightnessResult < -toolEpsilon) {
        vec3 hsv = rgb2hsv(result.rgb);
        hsv.z += hsv.z * brightnessResult / 3.0;
        result.rgb = hsv2rgb(hsv);
    }

    // fadeAmount
    if (abs(fadeAmountResult) > toolEpsilon) {
        result.rgb = fadeAdjust(result.rgb, fadeAmountResult);
    }
    //
    // vibrance
    if (abs(vibranceResult) > toolEpsilon) {
        float average = (result.r + result.g + result.b) / 3.0;
        float mx = max(result.r, max(result.g, result.b));
        float amt = (mx - average) * (-vibranceResult * 3.0);
        result.rgb = mix(result.rgb, vec3(mx), amt);
    }

    float satLuminance = dot(result.rgb, satLuminanceWeighting);
    vec3 greyScaleColor = vec3(satLuminance);
    result = vec4(mix(greyScaleColor, result.rgb, (1.0 + saturationResult) * (1. + dSaturation)), result.a);
    // saturation
    //    if (abs(saturation) > toolEpsilon) {
    //        float satLuminance = dot(result.rgb, satLuminanceWeighting);
    //        vec3 greyScaleColor = vec3(satLuminance);
    //        result = vec4(mix(greyScaleColor, result.rgb, saturationResult + 1.0), result.a);
    //    }
    //    // warmth
    if (abs(warmthResult) > toolEpsilon) {
        if (warmthResult > 0.0) {
            result.rgb = result.rgb + vec3(0.8, 0.8, 0.) * (warmthResult * alpha / 4.);
        } else {
            result.rgb = result.rgb + vec3(0., 0., 1.0) * (-warmthResult * alpha / 2.);
        }
    }
    // green
    if (abs(green) > toolEpsilon) {
        result = vec4(result.r, result.g * (greenResult + 1.0), result.b, result.a);
    }

    gl_FragColor = result;
}
